/*
 * gc global variable provides access to GUI Composer infrastructure components and project information.
 * For more information, please see the Working with Javascript guide in the online help.
 */
var gc = gc || {};
gc.services = gc.services || {};

/*
*  Boilerplate code for creating computed data bindings
*/
document.addEventListener('gc-databind-ready', function() 
{
    /* 
	*   Add custom computed value databindings here, using the following method:
	*
    *   function gc.databind.registry.bind(targetBinding, modelBinding, [getter], [setter]);
	*
	*   param targetBinding - single binding string or expression, or array of binding strings for multi-way binding.
	*   param modelBinding - single binding string or expression, or array of binding strings for multi-way binding.
	*   param getter - (optional) - custom getter function for computing the targetBinding value(s) based on modelBinding value(s).
	*   param setter - (optional) - custom setter function for computing the modelBinding value(s) based on targetBinding value(s).
    */
	
	// For example, a simple computed values based on simple expression
	// gc.databind.registry.bind('widget.id.propertyName', "targetVariable == 1 ? 'binding is one' : 'binding is not one'");
	
	// Or a custom two-way binding with custome getter and setter functions.  (setter is optional)  (getter only indicates one-way binding)
	// gc.databind.registry.bind('widget.id.propertyName', "targetVariable", function(value) { return value*5/9 + 32; }, function(value) { (return value-32)*9/5; });
	
	// Event 1 to n bindings
	/* 
    gc.databind.registry.bind('widget.date.value', 
        // dependant bindings needed in order to compute the date, in name/value pairs.
        {
            weekday: 'widget.dayOfWeek.selectedText',
            day: 'widget.dayOfMonth.value',
            month: 'widget.month.selectedText',
            year: 'widget.year.value'
        }, 
        // getter for date computation
        function(values) 
        {
            // compute and return the string value to bind to the widget with id 'date'
            return values.weekday + ', ' + values.month + ' ' + values.day + ', ' + values.year;
        }
    ); 
	*/
});

/*
*  Boilerplate code for creating custom actions
*/
document.addEventListener('gc-nav-ready', function() 
{
    /* 
	*   Add custom actions for menu items using the following api:
	*
    *   function gc.nav.registryAction(id, runable, [isAvailable], [isVisible]);
	*
	*   param id - uniquely identifies the action, and should correspond to the action property of the menuaction widget.
	*   param runable - function that performs the custom action.
	*   param isAvailable - (optional) - function called when the menu action is about to appear.  Return false to disable the action, or true to enable it.
	*   param isVisible - (optional) - function called when the menu action is about to appear.  Return false to hide the action, or true to make it visible.
    */
	
	// For example,
	// gc.nav.registerAction('myCustomCloseAction', function() { window.close(); }, function() { return true; }, function() { return true; });
	
	// Alternatively, to programmatically disable a menu action at any time use:
	// gc.nav.disableAction('myCustomCloseAction);    then enable it again using:  gc.nav.enableAction('myCustomCloseAction'); 
});


/*
*  Boilerplate code for working with components in the application gist
*/

var rc_status = false;      // if true, the power button is on
var lastBtnPressed = null;  // used to turn off a momentary button after it has been pressed
var lastTimestamp =  Date.now();

var analyse_command = function(cmd_name){
        var type = cmd_name >> 8
        var value = cmd_name & 0x00ff
        var newTimestamp = Date.now();
        var elapsedTime = newTimestamp - lastTimestamp;
        lastTimestamp = newTimestamp;
        console.log("elapsed time = "+elapsedTime+"ms:  type=", type+", value="+value);
        var btnName = null;
        switch (type) {
            case 0x05:
                console.log("Received volume command: isOn="+rc_status+", volume = "+value);
                //DON'T CASE THE ON/OFF BUTTON STATUS: if (rc_status) {
                    /* self.canvas.after.clear()
                    with (self.canvas.after) {
                        Color(0, 1, 0, 1)
                        d_led = 13.5
                        Ellipse(pos = (273, 525), size = (d_led, d_led))
                    }
                    */
                    var vol_slider = document.querySelector('#vol_slider');
                    vol_slider.value = value
                //}
                break;
            case 0x01:
                switch (value) {
                    case 0x11:  // btnNorth
                        btnName = 'btnNorth';
/*                        vol_value = 0;
                        d = 30;
                        p_x = 187.5;
                        p_y = 435;
                        */
                        break;
                    case 0x12:  //btnSouth
                        btnName = 'btnSouth';
                        /*
                        vol_value = 0;
                        d = 30;
                        p_x = 187.5;
                        p_y = 293;
                        */
                        break;
                    case 0x13:  //btnWest
                        btnName = 'btnWest';
                        /*
                        vol_value = 0;
                        d = 30;
                        p_x = 114.5;
                        p_y = 370;
                        */
                        break;
                    case 0x14:  //btnEast
                        btnName = 'btnEast';
                        /*
                        vol_value = 0;
                        d = 30;
                        p_x = 255;
                        p_y = 370;
                        */
                        break;
                    case 0x21:  //btnMiddle
                        btnName = 'btnMiddle';
                        /*
                        vol_value = 0;
                        d = 50;
                        p_x = 176.5;
                        p_y = 355;
                        */
                        break;
                    default: 
                    break;    
                }
                break;
            case 0x03:
                switch (value) {

                    case 0x00:  // btnOnOff
                        rc_status = !rc_status;
                        var btnOnOff = document.querySelector('#btnOnOff');
                        btnOnOff._buttonPressed = rc_status;
/*                        vol_value = 0;
                        d = 30;
                        p_x = 265;
                        p_y = 486;
                        */
                        break;
                    case 0x01:  //btnChPlus
                        btnName = 'btnChPlus';
                        
                        /* 
                        vol_value = 0;
                        d = 30;
                        p_x = 115.5;
                        p_y = 170;
                        */
                        break;
                    case 0x02:  //btnChMinus
                        btnName = 'btnChMinus';
                        /*
                        vol_value = 0;
                        d = 30;
                        p_x = 115.5;
                        p_y = 124;
                        */
                        break;
                    case 0x03:  //btnA
                        btnName = 'btnA';
                        /*
                        vol_value = 0;
                        d = 30;
                        p_x = 115.5;
                        p_y = 214;
                        */
                        break;
                    case 0x04:  //btnB
                        btnName = 'btnB';
                        /*vol_value = 0;
                        d = 30;
                        p_x = 184.5;
                        p_y = 214;
                        */
                        break;
                    case 0x05:  //btnC
                        btnName = 'btnC';
                        /*
                        vol_value = 0;
                        d = 30;
                        p_x = 254;
                        p_y = 214;
                        */
                        break;
                    case 0x06:  //btnInput
                        btnName = 'btnInput';
                        /*
                        vol_value = 0;
                        d = 30;
                        p_x = 254;
                        p_y = 170;
                        */
                        break;
                    case 0x07:  //btnMute
                        btnName = 'btnMute';
                        /*
                        vol_value = 0;
                        d = 30;
                        p_x = 254;
                        p_y = 124;
                        */
                        break;
                    default:
                        break;
                }
                break;
            default: 
                console.log("unrecognized command.");
                break;
        }
        if (btnName !== null){
            var btn = document.querySelector('#'+btnName);
            if (btn){
                console.log("Turning on "+btn.id);
                lastBtnPressed = btn;
                btn._buttonPressed = true;
            }
            // Wait 150 ms and then turn off the button that was just pressed.
            window.setTimeout(function(){
                if (lastBtnPressed !== null){
                    var newTimestamp1 = Date.now();
                    var elapsedTime1 = newTimestamp1 - lastTimestamp;
                    lastTimestamp = newTimestamp1;
                    console.log("elasped time: "+elapsedTime1+"ms.  Turning off "+lastBtnPressed.id);
                    lastBtnPressed._buttonPressed = false;
                }
                lastBtnPressed = null;
            },250);
        }
        
  }



var initComplete = false;
var templateObj;
var isCmdByte = false;
var lastCmd = 0;
var lastValue = 0;

// Wait for DOMContentLoaded event before trying to access the application template
var init = function() {
    templateObj = document.querySelector('#template_obj');

    
    templateObj.serialPortOutputHdlr = {
        text: function (rxMsg,buffer) {
            var cmdWord;
            var rxByte;
            var i=0;
            var done = false;
            // assume that we always receive pairs of bytes corresponding to command / value
            isCmdByte = false;
            while (i < buffer.length){
                rxByte = buffer[i++];
                if (isCmdByte){
                    lastCmd = rxByte;
                } else {
                    lastValue = rxByte;
                }
                var ok = false;
                if (isCmdByte){
                    switch (lastCmd){
                        case 0x1: 
                            ok = true;
                            break;
                        case 0x3: 
                            ok = true;
                            break;
                        case 0x5: 
                            ok = true;
                            break;
                        default: 
                        isCmdByte = false;
                        break;
                    }
                }
                isCmdByte = !isCmdByte;
                if (ok){
                    cmdWord = (((lastCmd & 0xff)<< 8) | (lastValue & 0xff));
                    analyse_command(cmdWord);
                }
            }
        }
    }

    // Wait for the template to fire a dom-change event to indicate that it has been 'stamped'
    // before trying to access components in the application.
	templateObj.addEventListener('dom-change',function(){
	    if (initComplete) return;
	    this.async(function(){
    	    initComplete = true;
    	    console.log("Application template has been stamped.");
  	        // Now that the template has been stamped, you can use 'automatic node finding' $ syntax to access widgets.
  	        // e.g. to access a widget with an id of 'widget_id' you can use templateObj.$.widgetId
	        templateObj.backplane = gc.services['ti-core-backplane'];
            templateObj.serialIO = gc.services['ti-service-serialio']
            templateObj.serialIO.serialPortTextOutputHdlr = templateObj.serialPortOutputHdlr;


        },1);

	});
};

templateObj = document.querySelector('#template_obj');
if (templateObj) {
    init();
} else {
    document.addEventListener('DOMContentLoaded',init.bind(this))
}


